from __future__ import annotations

from dataclasses import dataclass


@dataclass
class FsOptions:
    cache_path: str | None = None
    http_cache: bool | None = None
    control_path: str | None = None
    listen_events: bool | None = None



from time import sleep

from plexfuse.control.ControlListener import ControlListener
from plexfuse.fs.PlexFS import PlexFS
from plexfuse.plex.PlexApi import PlexApi
from plexfuse.vfs.Control import Control
from plexfuse.vfs.PlexVFS import PlexVFS


def test_socket_control():
    p = PlexApi()
    fs = PlexFS()
    vfs = PlexVFS(p, fs)
    c = Control(p, fs, vfs)
    s = ControlListener("/tmp/ControlListener.sock", c)
    s.start()

    try:
        for i in range(100):
            print(f"Main thread idling: {i}...")
            sleep(10)
    except KeyboardInterrupt as e:
        print(e)
    finally:
        s.stop()
    print("Main thread exiting")


if __name__ == "__main__":
    test_socket_control()



from __future__ import annotations

import os
import os.path
import socket
from threading import Thread
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from plexfuse.vfs.Control import Control


class ControlListener:
    def __init__(self, path: str, control: Control):
        self.path = path
        self.control = control
        self.thread = None
        self.socket = None

    def start(self):
        self.socket = self.listen_socket()
        self.thread = self.create_thread(self.handle)

        return self

    def stop(self):
        self.close_socket()
        self.stop_thread()

    @staticmethod
    def create_thread(handler):
        thread = Thread(target=handler, daemon=True)
        thread.start()
        return thread

    def stop_thread(self):
        if self.thread is None:
            return

        if self.thread.is_alive():
            self.thread.join()

        self.thread = None

    def listen_socket(self):
        if os.path.exists(self.path):
            os.unlink(self.path)

        server = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        server.bind(self.path)
        server.listen()

        return server

    def close_socket(self):
        socket = self.socket
        self.socket = None
        socket.close()
        if os.path.exists(self.path):
            os.unlink(self.path)

    def handle(self):
        while True:
            try:
                conn, addr = self.socket.accept()
            except ConnectionAbortedError as e:
                print(e)
                # Server is shutting down
                if not self.socket:
                    print("No more socket, exiting")
                    return
                continue

            datagram = conn.recv(128)
            if datagram:
                tokens = datagram.decode().strip().split()
                if tokens[0] in self.control.commands:
                    response = self.control.action(tokens[0])
                    conn.send(response)
                else:
                    conn.send(b"Bye\n")
            conn.close()



from __future__ import annotations

import stat

from plexfuse.vfs.entry.AttrEntry import AttrEntry


class ControlSockEntry(AttrEntry):
    def __init__(self, name: str, control_path: str):
        self.name = name
        self.size = 0
        self.link = control_path

    @property
    def attr(self):
        return {
            "st_mode": stat.S_IFLNK | 0o644
        }



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from plexfuse.plex.PlexApi import PlexApi


class Monitor:
    def __init__(self, plex: PlexApi):
        self.plex = plex
        self.notifier = None

    def start(self):
        # Perform self-test, like PlexAPI
        try:
            import websocket
        except ImportError:
            print("Can't use Monitor without websocket")
            return None

        del websocket

        self.notifier = self.plex.server.startAlertListener(
            callback=self.event_handler,
            callbackError=self.event_handler,
        )
        print("Started notifier:", self.notifier)
        self.event_handler("test")

        return self

    def stop(self):
        if not self.notifier:
            return

        print("Stopping notifier:", self.notifier)
        self.notifier.stop()

        if self.notifier.is_alive():
            self.notifier.join(timeout=10)
            if self.notifier.is_alive():
                print("Thread for notifier is still alive:", self.notifier)

        self.notifier = None

    @property
    def is_alive(self):
        return self.notifier and self.notifier.is_alive()

    @staticmethod
    def event_handler(data):
        print("plex event: ", data)
